% extract multiple linear motion models with RANSAC
function [models label_pts] = RANSAC_linearMotion(g_alpha, pos, h, w, nb_model)

dis_thr = 0.01;         % inlier threshold for RANSAC
nb_trial = 2000;        % number of trial 

nb_pts = size(g_alpha, 1);
models = zeros(nb_model, 2);

label_pts = zeros(nb_pts,1);    % the index of the best model for each gradient point
idx_valid = 1:nb_pts;           % points not assigned to any model so far
for active_label = 1:nb_model
	%visualizeHough(g_alpha(idx_valid,:), 20, 4);
    nb_inlier = 0;
    nb_pts_valid = numel(idx_valid);
    for tt = 1:nb_trial
        i1 = idx_valid(ceil(rand(1) * nb_pts_valid));
        i2 = idx_valid(ceil(rand(1) * nb_pts_valid));
        % Ax = b
        b = [1 1]';
        A = [g_alpha(i1,1) g_alpha(i1,2); g_alpha(i2,1) g_alpha(i2,2)];
        x = A\b;
        xn = norm(x);        
        % two possibilities for each line
        dis1 = abs(sum(repmat(x',nb_pts,1).*g_alpha, 2) - 1) ./ xn;
        dis2 = abs(sum(repmat(-x',nb_pts,1).*g_alpha, 2) - 1) ./ xn;
        dis = min(dis1, dis2);
        nb_inlier_this = sum( dis<dis_thr & label_pts==0 );
        if ( nb_inlier_this > nb_inlier )
            nb_inlier = nb_inlier_this;
            best_model = x;
            label_best_idx = ( dis<dis_thr & label_pts==0 );
        end
    end
    label_pts = label_pts + label_best_idx * active_label;
    models(active_label,:) = best_model;
    
    idx_valid = setdiff(idx_valid, find(label_best_idx));    
end

% refine multiple models jointly
for iter = 1:3
    % refine the label
    all_dis = zeros(nb_model+1, nb_pts);    % distance to each extracted model, nb_model+1 is for all outliers
    label_pn = zeros(nb_model, nb_pts);     % indicator of using +1 or -1
    for i = 1: nb_model
        x = models(i,:);
        nx = norm(x);
        dis1 = abs(sum(repmat(x,nb_pts,1).*g_alpha, 2) - 1) ./ nx;
        dis2 = abs(sum(repmat(-x,nb_pts,1).*g_alpha, 2) - 1) ./ nx;
        [all_dis(i,:) label_pn(i,:)]= min([dis1';dis2']);
    end
    all_dis(nb_model+1, :) = dis_thr;
    [m label_pts] = min(all_dis);

    % refine the model
    for i=1:nb_model
        nb_pts_this_model = sum(label_pts==i);
        b = ones(nb_pts_this_model, 1);
        A = g_alpha(label_pts==i, :);
        label_pn_this = label_pn(i,label_pts==i);
        A = A .* repmat((2*(label_pn_this==1)-1)', 1, 2);
        models(i,:) = (A\b)';
    end
end

% visualization of the pixel labels
% valid_mask = zeros(h,w);
% valid_mask(sub2ind([h w], pos(:,1), pos(:,2) )) = label_pts;
% figure; imshow(valid_mask,[]); colormap(jet);