% motion model extraction
% nb_model:         number of linear motion model          
% consist_thr:      local consistency threshold, typically 20 for synthetic image, 100 for real image

function processAlpha(imgn, nb_model, consist_thr)

winsz           = 30; % size of valid region of each block
border          = 10; % overlaping border width of neighboring block, so the full block size is usually winsz+2*border

img_blur = imread([imgn '.png']);
figure; imshow(img_blur); title('input image');
fn = ['./alpha/all_alpha_' imgn];
load(fn, 'all_alpha');

[h w c]         = size(img_blur);

%=====================================
% extract the pixels with local consistentcy
%=====================================
valid_mask      = zeros(h,w);       % mask of local consistent region
g_alpha_all     = zeros(h*w, 2);    % gradient of alpha, y gradient first, for the extracted consistent pixels
g_dist_all      = zeros(h*w, 1);    % local inconsistency measurement, for the extracted consistent pixels
g_pos_all       = zeros(h*w, 2);    % pixel position of the extracted consistent pixels
g_full_image    = zeros(h,w,2);     % gradient of alpha, for the whole image
g_nb            = 0;                % number of the extracted consistent pixels
alpha_all       = zeros(h, w);      % alpha channel visualization, only show the valid region of each block
    
for startx = 1:winsz:w-border       % start position of the valid region of each block
    for starty = 1:winsz:h-border
        % parameters of the valid region of the current block
        valid_y = [starty, min(h,starty+winsz-1)];
        valid_h = valid_y(2)+1-valid_y(1);
        valid_x = [startx, min(w,startx+winsz-1)];
        valid_w = valid_x(2)+1-valid_x(1);
        % parameters of the entire current block
        entire_y = [max(starty-border,1), min(h, valid_y(2)+border)];
        entire_h = entire_y(2)+1-entire_y(1);
        entire_x = [max(startx-border,1), min(w, valid_x(2)+border)];
        entire_w = entire_x(2)+1-entire_x(1);

        %patch = img_blur(entire_y(1):entire_y(2), entire_x(1):entire_x(2),:);
        this_alpha = squeeze(all_alpha{ceil((starty-1)/winsz)+1}{ceil((startx-1)/winsz)+1}{1});

        sy_this = valid_y(1) - entire_y(1) + 1;
        sx_this = valid_x(1) - entire_x(1) + 1;
        alpha_all(valid_y(1):valid_y(2), valid_x(1):valid_x(2)) = this_alpha(sy_this:sy_this+valid_h-1, sx_this:sx_this+valid_w-1);
        
        if ( sy_this > 1 )
            g_full_image(valid_y(1):valid_y(2), valid_x(1):valid_x(2),1) = ...
                this_alpha(sy_this:sy_this+valid_h-1, sx_this:sx_this+valid_w-1)...
                -this_alpha(sy_this-1:sy_this+valid_h-2, sx_this:sx_this+valid_w-1);
        else
            g_full_image(valid_y(1)+1:valid_y(2), valid_x(1):valid_x(2),1) = ...
                this_alpha(sy_this+1:sy_this+valid_h-1, sx_this:sx_this+valid_w-1)...
                -this_alpha(sy_this:sy_this+valid_h-2, sx_this:sx_this+valid_w-1);
        end
        if ( sx_this > 1 )
            g_full_image(valid_y(1):valid_y(2), valid_x(1):valid_x(2),2) = ...
                this_alpha(sy_this:sy_this+valid_h-1, sx_this:sx_this+valid_w-1)...
                -this_alpha(sy_this:sy_this+valid_h-1, sx_this-1:sx_this+valid_w-2);
        else
            g_full_image(valid_y(1):valid_y(2), valid_x(1)+1:valid_x(2),2) = ...
                this_alpha(sy_this:sy_this+valid_h-1, sx_this+1:sx_this+valid_w-1)...
                -this_alpha(sy_this:sy_this+valid_h-1, sx_this:sx_this+valid_w-2);
        end

        [g_alpha, g_dist, g_pos, g_mask] = getLocalConsistAlphaGrad(this_alpha, consist_thr, border); 
        nb_this = size(g_alpha, 1);
        g_alpha_all(g_nb+1:g_nb+nb_this,:) = g_alpha;
        g_dist_all(g_nb+1:g_nb+nb_this) = g_dist;
        g_pos_all(g_nb+1:g_nb+nb_this,:) = g_pos + repmat([starty startx], nb_this, 1) - 1;
        if ( starty == 1)
            g_pos_all(g_nb+1:g_nb+nb_this,1) = g_pos_all(g_nb+1:g_nb+nb_this,1) + border;
        end
        if ( startx == 1)
            g_pos_all(g_nb+1:g_nb+nb_this,2) = g_pos_all(g_nb+1:g_nb+nb_this,2) + border;
        end
        
        valid_mask(entire_y(1)+border:entire_y(2)-border, entire_x(1)+border:entire_x(2)-border) ...
            = g_mask;
        
        g_nb = g_nb + nb_this;
    end
end
g_alpha_all = real(g_alpha_all(1:g_nb,:));
g_dist_all = g_dist_all(1:g_nb,:);
g_pos_all = g_pos_all(1:g_nb,:);

g_pos_all = g_pos_all - border; % for old

% visualization of some intermediate results
figure; imshow(alpha_all, [0 1]); title('alpha channel');
ss = zeros(h,w);
ss(sub2ind([h w], g_pos_all(:,1), g_pos_all(:,2))) = g_alpha_all(:,1);
figure; imshow(abs(ss), [-0.5 0.5]); title('y gradient of alpha channel (consistent pixel only)');
visualizeHough(g_alpha_all, 20, 4);

[models label] = RANSAC_linearMotion(g_alpha_all, g_pos_all, h, w, nb_model);
models

%============================
% visualize the model error
%============================
dy = squeeze(g_full_image(:,:,1));
dx = squeeze(g_full_image(:,:,2));
nb_model = size(models, 1);
for i=1:nb_model 
    modeln = norm(models(i,:));
    tempval = dx*models(i,2) + dy*models(i,1);
    this_dis = min(abs(tempval-1), abs(tempval+1)) / modeln;
    titleStr = sprintf('distance to model #%d (all pixels)', i);
    figure; imshow(this_dis,[0 .1]); title(titleStr);
end